var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Component, Input, ViewChild } from '@angular/core';
import viser from 'viser';
import { ChartContext } from './chartService';
function firstLowerCase(str) {
    return str.replace(/^\S/, function (s) {
        return s.toLowerCase();
    });
}
function generateRandomNum() {
    return (Math.floor(new Date().getTime() + Math.random() * 10000)).toString();
}
function isOwnEmpty(obj) {
    for (var name_1 in obj) {
        if (obj.hasOwnProperty(name_1)) {
            return false;
        }
    }
    return true;
}
function retain(obj, attr) {
    var newObj = Object.create(null);
    for (var item in obj) {
        if (obj.hasOwnProperty(item)) {
            var arrAttr = Array.isArray(attr) ? attr : [attr];
            if (arrAttr.indexOf(item) >= 0) {
                newObj[item] = obj[item];
            }
        }
    }
    return newObj;
}
function omit(obj, attr) {
    var newObj = Object.create(null);
    for (var item in obj) {
        if (obj.hasOwnProperty(item)) {
            var arrAttr = Array.isArray(attr) ? attr : [attr];
            if (arrAttr.indexOf(item) < 0) {
                newObj[item] = obj[item];
            }
        }
    }
    return newObj;
}
var Chart = (function () {
    function Chart(context) {
        this.context = context;
        this.config = {};
        this.views = {};
        this.chart = null;
        this.viewId = context.viewId;
        this.context = context;
    }
    Chart.prototype.combineViewConfig = function (props, config) {
        if (props.data) {
            config.data = props.data;
        }
        if (props.dataMapping) {
            config.dataMapping = props.dataMapping;
        }
        if (props.dataPre) {
            config.dataPre = props.dataPre;
        }
        if (props.dataView) {
            config.dataView = props.dataView;
        }
        if (props.scale) {
            config.scale = props.scale;
        }
    };
    Chart.prototype.combineChartConfig = function (props, config) {
        var chartRetain = [
            'height', 'width', 'animate', 'forceFit',
            'background', 'plotBackground', 'padding',
        ];
        config.chart = retain(props, chartRetain);
    };
    Chart.prototype.combineContentConfig = function (displayName, props, config) {
        var nameLowerCase = displayName.toLowerCase();
        var regSeries = [
            'pie',
            'sector',
            'line',
            'smoothline',
            'dashline',
            'area',
            'stackarea',
            'smootharea',
            'bar',
            'stackbar',
            'dodgebar',
            'point',
            'waterfall',
            'funnel',
            'pyramid',
            'radialbar',
            'schema',
            'box',
            'candle',
            'polygon',
            'contour',
            'heatmap',
            'edge'
        ];
        if (regSeries.indexOf(nameLowerCase) >= 0) {
            if (!config.series) {
                config.series = [];
            }
            config.series.push(__assign({ quickType: firstLowerCase(displayName) }, props));
        }
        else if (isOwnEmpty(props)) {
            config[nameLowerCase] = true;
        }
        else if (nameLowerCase === 'axis') {
            if (!config.axis) {
                config.axis = [];
            }
            config.axis.push(props);
        }
        else if (nameLowerCase === 'series') {
            if (!config.series) {
                config.series = [];
            }
            config.series.push(props);
        }
        else if (nameLowerCase === 'guide') {
            if (!config.guide) {
                config.guide = [];
            }
            config.guide.push(props);
        }
        else {
            config[nameLowerCase] = props;
        }
        return config;
    };
    Chart.prototype.ngAfterViewInit = function () {
        if (this.chart) {
            this.chart.destroy();
        }
        this.initChart();
    };
    Chart.prototype.getProps = function (allProps) {
        var strippingProperties = ['chart', 'chartDiv', 'config', 'context', 'viewId', 'views',
            'constructor', 'combineViewConfig', 'combineChartConfig', 'combineContentConfig',
            'ngAfterViewInit', 'getProps', 'getViewChartConfig', 'initChart', 'ngOnChanges', 'renderChart'];
        if (allProps) {
            var properties = {};
            for (var key in allProps) {
                if (strippingProperties.indexOf(key) === -1) {
                    properties[key] = allProps[key];
                }
            }
            return properties;
        }
        return allProps;
    };
    Chart.prototype.getViewChartConfig = function (config) {
        var chartProperties = ['forceFit', 'height', 'width', 'container'];
        var chart = {};
        if (config.chart) {
            for (var key in config.chart) {
                if (chartProperties.indexOf(key) > -1) {
                    chart[key] = config.chart[key];
                }
            }
        }
        return chart;
    };
    Chart.prototype.initChart = function (rerender) {
        var name = this.constructor.name;
        var props = this.getProps(this);
        this.combineContentConfig(name, props, this.context.config);
        this.combineChartConfig(props, this.context.config);
        this.combineViewConfig(props, this.context.config);
        this.context.config.chart = this.getViewChartConfig(this.context.config);
        if (this.constructor.name === 'Chart') {
            this.renderChart(rerender);
        }
        else if (this.constructor.name === 'Facet') {
            this.context.config.facet.views = __assign({}, this.context.config.facet.views, this.context.config, { series: this.context.config.series && this.context.config.series[0] });
            delete this.context.config.series;
        }
    };
    Chart.prototype.ngOnChanges = function (changes) {
        if (!this.chart) {
            return;
        }
        this.initChart(true);
    };
    Chart.prototype.renderChart = function (rerender) {
        this.context.config.chart.container = this.chartDiv.nativeElement;
        if (rerender) {
            this.chart.repaint(this.context.config);
        }
        else {
            this.chart = viser(this.context.config);
        }
    };
    __decorate([
        Input()
    ], Chart.prototype, "data");
    __decorate([
        Input()
    ], Chart.prototype, "dataMapping");
    __decorate([
        Input()
    ], Chart.prototype, "dataPre");
    __decorate([
        Input()
    ], Chart.prototype, "height");
    __decorate([
        Input()
    ], Chart.prototype, "width");
    __decorate([
        Input()
    ], Chart.prototype, "animate");
    __decorate([
        Input()
    ], Chart.prototype, "forceFit");
    __decorate([
        Input()
    ], Chart.prototype, "background");
    __decorate([
        Input()
    ], Chart.prototype, "plotBackground");
    __decorate([
        Input()
    ], Chart.prototype, "padding");
    __decorate([
        Input()
    ], Chart.prototype, "scale");
    __decorate([
        Input()
    ], Chart.prototype, "dataView");
    __decorate([
        ViewChild('chartDom')
    ], Chart.prototype, "chartDiv");
    Chart = __decorate([
        Component({
            providers: [ChartContext],
            selector: 'Chart',
            template: "<div #chartDom></div>"
        })
    ], Chart);
    return Chart;
}());
export { Chart };
//# sourceMappingURL=Chart.js.map